/**
 * @file
 * test.adjustColorBrightness node implementation.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

#include "VuoImageRenderer.h"

VuoModuleMetadata({
                     "title" : "Water Simulation Filter",
                     "description" : "Creates a water simulation, based on GLSL Sandbox example by @notlion, @alteredq. 2014, George Toledo.",
					 "keywords" : [ ],
					 "version" : "1.0.0",
					 "dependencies" : [
						 "VuoImageRenderer"
					 ],
					 "node": {
						 "isInterface" : false
					 }
				 });

#include "node.h"

static const char * fragmentShaderSource = VUOSHADER_GLSL_SOURCE(120,
	// Inputs
	uniform sampler2D texture;
    uniform sampler2D backbuffer;

    uniform float ResX;
    uniform float ResY;
    uniform vec4 color;
	varying vec4 fragmentTextureCoordinate;

    float getSpring(float r, vec2 pos, float power){
      return (texture2D(backbuffer, pos).r - r) * power;
    }


	void main()
	{
		vec4 frag = texture2D(texture, fragmentTextureCoordinate.xy);
        vec2 pos = fragmentTextureCoordinate.xy;

       // vec2 pos = gl_FragCoord.xy / resolution;
        vec2 resolution = vec2(ResX,ResY);
        vec2 pixel = 4. / resolution;
        float aspect = resolution.x / resolution.y;

        vec4 texel_prev = texture2D(backbuffer, pos);
        vec4 tex = texture2D(texture, pos);

          float r_prev = texel_prev.r;
          float power = .5;

          float vel = texel_prev.a - 0.5;
          vel += getSpring(r_prev, pos + pixel * vec2(2, 3), 0.0022411859348636983 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(1, 3), 0.004759786021770571 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(0, 3), 0.005681818181818182 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-1, 3), 0.004759786021770571 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-2, 3), 0.0022411859348636983 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(3, 2), 0.0022411859348636983 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(2, 2), 0.0066566640639421 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(1, 2), 0.010022341036933013 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(0, 2), 0.011363636363636364 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-1, 2), 0.010022341036933013 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-2, 2), 0.0066566640639421 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-3, 2), 0.0022411859348636983 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(3, 1), 0.004759786021770571 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(2, 1), 0.010022341036933013 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(1, 1), 0.014691968395607415 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(0, 1), 0.017045454545454544 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-1, 1), 0.014691968395607415 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-2, 1), 0.010022341036933013 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-3, 1), 0.004759786021770571 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(3, 0), 0.005681818181818182 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(2, 0), 0.011363636363636364 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(1, 0), 0.017045454545454544 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-1, 0), 0.017045454545454544 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-2, 0), 0.011363636363636364 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-3, 0), 0.005681818181818182 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(3, -1), 0.004759786021770571 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(2, -1), 0.010022341036933013 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(1, -1), 0.014691968395607415 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(0, -1), 0.017045454545454544 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-1, -1), 0.014691968395607415 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-2, -1), 0.010022341036933013 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-3, -1), 0.004759786021770571 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(3, -2), 0.0022411859348636983 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(2, -2), 0.0066566640639421 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(1, -2), 0.010022341036933013 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(0, -2), 0.011363636363636364 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-1, -2), 0.010022341036933013 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-2, -2), 0.0066566640639421 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-3, -2), 0.0022411859348636983 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(2, -3), 0.0022411859348636983 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(1, -3), 0.004759786021770571 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(0, -3), 0.005681818181818182 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-1, -3), 0.004759786021770571 * power);
          vel += getSpring(r_prev, pos + pixel * vec2(-2, -3), 0.0022411859348636983 * power);

          vel += (.25 - r_prev) * .025 * power;

          vel += max(0.,  length(tex));//-1.5);

          gl_FragColor = vec4(texel_prev.rgb + vel, vel * .98 + .5);
          gl_FragColor *= vec4(color);
    }

);
struct nodeInstanceData
{
	VuoShader shader;
	VuoGlContext glContext;
	VuoImageRenderer imageRenderer;
};

struct nodeInstanceData * nodeInstanceInit(void)
{
	struct nodeInstanceData * instance = (struct nodeInstanceData *)malloc(sizeof(struct nodeInstanceData));
	VuoRegister(instance, free);

	instance->glContext = VuoGlContext_use();

    instance->shader = VuoShader_make("Water Simulation Filter");
	VuoShader_addSource(instance->shader, VuoMesh_IndividualTriangles, NULL, NULL, fragmentShaderSource);
	VuoRetain(instance->shader);

	instance->imageRenderer = VuoImageRenderer_make(instance->glContext);
	VuoRetain(instance->imageRenderer);

	return instance;
}

void nodeInstanceEvent
(
		VuoInstanceData(struct nodeInstanceData *) instance,
		VuoInputData(VuoImage) image,
        VuoInputData(VuoImage) backbuffer,
        VuoInputData(VuoColor, {"default":{"r":1,"g":1,"b":1,"a":1}}) color,

		VuoOutputData(VuoImage) adjustedImage
)
{
	if (! image)
		return;

	// Feed parameters to the shader.
	VuoShader_setUniform_VuoImage((*instance)->shader, "texture", image);
    VuoShader_setUniform_VuoImage((*instance)->shader, "backbuffer", backbuffer);
    VuoShader_setUniform_VuoColor((*instance)->shader, "color", color);

    VuoShader_setUniform_VuoReal ((*instance)->shader, "ResX",   image->pixelsWide);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "ResY",   image->pixelsHigh);


	// Render.
	*adjustedImage = VuoImageRenderer_draw((*instance)->imageRenderer, (*instance)->shader, image->pixelsWide, image->pixelsHigh, VuoImage_getColorDepth(image));
}

void nodeInstanceFini(VuoInstanceData(struct nodeInstanceData *) instance)
{
	VuoRelease((*instance)->shader);
	VuoRelease((*instance)->imageRenderer);
	VuoGlContext_disuse((*instance)->glContext);
}
